import WebSocket, { ServerOptions } from 'ws';

type WithId<T> = T & {
    id?: number | string;
};

interface BaseErrorResponse {
    error: {
        code: ErrorCodes | number;
        message: string;
    };
}
type ErrorResponse = WithId<BaseErrorResponse>;
declare enum ErrorCodes {
    ParseError = -32700,
    InvalidRequest = -32600,
    MethodNotFound = -32601,
    InvalidParams = -32602,
    InternalError = -32603,
    ResponseError = -32500
}

type ResponseResult = unknown;
interface BaseResponse {
    result: ResponseResult;
}
type Response = WithId<BaseResponse>;

interface WebSocketClient extends WebSocket {
    isAlive: boolean;
    sendResponse: (response: Response | ErrorResponse) => void;
}

type RequestParams = unknown;
interface BaseRequest {
    method: string;
    params?: RequestParams;
}
type Request = WithId<BaseRequest>;

type PromiseOr<T> = Promise<T> | T;

declare abstract class AbstractRequest {
    abstract readonly method: string;
    abstract invoke(params: RequestParams | undefined, ws: WebSocketClient): PromiseOr<ResponseResult>;
}

declare class Server {
    private debug;
    private webSocketServer;
    private requestsManager;
    constructor(options: ServerOptions, debug?: boolean);
    registerRequest(request: AbstractRequest): void;
    private connectHandler;
}

declare class ResponseError extends Error {
    private readonly code;
    constructor(message?: string, code?: number);
    toJSON(): BaseErrorResponse;
}

declare class InvalidParamsError extends ResponseError {
    constructor(message?: string, code?: number);
}

export { AbstractRequest, type BaseErrorResponse, type BaseRequest, type BaseResponse, ErrorCodes, type ErrorResponse, InvalidParamsError, type Request, type RequestParams, type Response, ResponseError, type ResponseResult, Server, type WebSocketClient };
